import React__default from "react";
import styles from "../../styles/ui.module.css.js";
import { useCodeBlockEditorContext } from "../codeblock/CodeBlockNode.js";
import { readOnly$ } from "../core/index.js";
import { useCellValues } from "@mdxeditor/gurx";
import { EditorState } from "@codemirror/state";
import { lineNumbers, EditorView } from "@codemirror/view";
import { basicLight } from "cm6-theme-basic-light";
import { basicSetup } from "codemirror";
import { languages } from "@codemirror/language-data";
import { useCodeMirrorRef } from "../sandpack/useCodeMirrorRef.js";
import { codeMirrorExtensions$, codeMirrorAutoLoadLanguageSupport$ } from "./index.js";
const COMMON_STATE_CONFIG_EXTENSIONS = [];
const CodeMirrorEditor = ({ language, nodeKey, code, focusEmitter }) => {
  const [readOnly, codeMirrorExtensions, autoLoadLanguageSupport] = useCellValues(
    readOnly$,
    codeMirrorExtensions$,
    codeMirrorAutoLoadLanguageSupport$
  );
  const codeMirrorRef = useCodeMirrorRef(nodeKey, "codeblock", language, focusEmitter);
  const { setCode } = useCodeBlockEditorContext();
  const editorViewRef = React__default.useRef(null);
  const elRef = React__default.useRef(null);
  const setCodeRef = React__default.useRef(setCode);
  setCodeRef.current = setCode;
  codeMirrorRef.current = {
    getCodemirror: () => editorViewRef.current
  };
  React__default.useEffect(() => {
    void (async () => {
      const extensions = [
        ...codeMirrorExtensions,
        basicSetup,
        basicLight,
        lineNumbers(),
        EditorView.lineWrapping,
        EditorView.updateListener.of(({ state }) => {
          setCodeRef.current(state.doc.toString());
        })
      ];
      if (readOnly) {
        extensions.push(EditorState.readOnly.of(true));
      }
      if (language !== "" && autoLoadLanguageSupport) {
        const languageData = languages.find((l) => {
          return l.name === language || l.alias.includes(language) || l.extensions.includes(language);
        });
        if (languageData) {
          try {
            const languageSupport = await languageData.load();
            extensions.push(languageSupport.extension);
          } catch (e) {
            console.warn("failed to load language support for", language);
          }
        }
      }
      elRef.current.innerHTML = "";
      editorViewRef.current = new EditorView({
        parent: elRef.current,
        state: EditorState.create({ doc: code, extensions })
      });
    })();
    return () => {
      var _a;
      (_a = editorViewRef.current) == null ? void 0 : _a.destroy();
      editorViewRef.current = null;
    };
  }, [readOnly, language]);
  return /* @__PURE__ */ React__default.createElement(
    "div",
    {
      className: styles.codeMirrorWrapper,
      onKeyDown: (e) => {
        e.stopPropagation();
      }
    },
    /* @__PURE__ */ React__default.createElement("div", { ref: elRef })
  );
};
export {
  COMMON_STATE_CONFIG_EXTENSIONS,
  CodeMirrorEditor
};
